#pragma once

#define BOOST_MPL_CFG_NO_PREPROCESSED_HEADERS
#define BOOST_MPL_LIMIT_VECTOR_SIZE 30

#include <string>

#ifdef __linux__
//#define MARK_UNUSED __attribute((unused))
#define MARK_UNUSED
#define TYPENAME typename
#else
#define MARK_UNUSED
#define TYPENAME
#endif

#ifndef __linux__
#ifndef __APPLE__
#pragma warning (disable: 4251)
#endif
#endif

namespace knx
{

//! reset
static const std::string COLOR_OFF="\033[0m"      ; // Text Reset
//! regular Colors
static const std::string Black="\033[0;30m"       ; // Black
static const std::string Red="\033[0;31m"         ; // Red
static const std::string Green="\033[0;32m"       ; // Green
static const std::string Yellow="\033[0;33m"      ; // Yellow
static const std::string Blue="\033[0;34m"        ; // Blue
static const std::string Purple="\033[0;35m"      ; // Purple
static const std::string Cyan="\033[0;36m"        ; // Cyan
static const std::string White="\033[0;37m"       ; // White
//! bold
static const std::string BBlack="\033[1;30m"      ; // Black
static const std::string BRed="\033[1;31m"        ; // Red
static const std::string BGreen="\033[1;32m"      ; // Green
static const std::string BYellow="\033[1;33m"     ; // Yellow
static const std::string BBlue="\033[1;34m"       ; // Blue
static const std::string BPurple="\033[1;35m"     ; // Purple
static const std::string BCyan="\033[1;36m"       ; // Cyan
static const std::string BWhite="\033[1;37m"      ; // White
//! underline
static const std::string UBlack="\033[4;30m"      ; // Black
static const std::string URed="\033[4;31m"        ; // Red
static const std::string UGreen="\033[4;32m"      ; // Green
static const std::string UYellow="\033[4;33m"     ; // Yellow
static const std::string UBlue="\033[4;34m"       ; // Blue
static const std::string UPurple="\033[4;35m"     ; // Purple
static const std::string UCyan="\033[4;36m"       ; // Cyan
static const std::string UWhite="\033[4;37m"      ; // White
//! background
static const std::string On_Black="\033[40m"      ; // Black
static const std::string On_Red="\033[41m"        ; // Red
static const std::string On_Green="\033[42m"      ; // Green
static const std::string On_Yellow="\033[43m"     ; // Yellow
static const std::string On_Blue="\033[44m"       ; // Blue
static const std::string On_Purple="\033[45m"     ; // Purple
static const std::string On_Cyan="\033[46m"       ; // Cyan
static const std::string On_White="\033[47m"      ; // White
//! high intensity
static const std::string IBlack="\033[0;90m"      ; // Black
static const std::string IRed="\033[0;91m"        ; // Red
static const std::string IGreen="\033[0;92m"      ; // Green
static const std::string IYellow="\033[0;93m"     ; // Yellow
static const std::string IBlue="\033[0;94m"       ; // Blue
static const std::string IPurple="\033[0;95m"     ; // Purple
static const std::string ICyan="\033[0;96m"       ; // Cyan
static const std::string IWhite="\033[0;97m"      ; // White
//! bold high intensity
static const std::string BIBlack="\033[1;90m"     ; // Black
static const std::string BIRed="\033[1;91m"       ; // Red
static const std::string BIGreen="\033[1;92m"     ; // Green
static const std::string BIYellow="\033[1;93m"    ; // Yellow
static const std::string BIBlue="\033[1;94m"      ; // Blue
static const std::string BIPurple="\033[1;95m"    ; // Purple
static const std::string BICyan="\033[1;96m"      ; // Cyan
static const std::string BIWhite="\033[1;97m"     ; // White
//! high intensity backgrounds
static const std::string On_IBlack="\033[0;100m"  ; // Black
static const std::string On_IRed="\033[0;101m"    ; // Red
static const std::string On_IGreen="\033[0;102m"  ; // Green
static const std::string On_IYellow="\033[0;103m" ; // Yellow
static const std::string On_IBlue="\033[0;104m"   ; // Blue
static const std::string On_IPurple="\033[0;105m" ; // Purple
static const std::string On_ICyan="\033[0;106m"   ; // Cyan
static const std::string On_IWhite="\033[0;107m"  ; // Whiteolor_Off="\033[0m"      ; // Text Reset

//! deine colors
static const std::string COLOR_RESET = COLOR_OFF;
static const std::string COLOR_INFO = Green;
static const std::string COLOR_WARNING = Yellow;
static const std::string COLOR_ERROR = Red;
static const std::string COLOR_SENDING = BBlue;
static const std::string COLOR_RECEIVING = BYellow;


} // end namespace

//! cut out the last filename
#define SIMPLE_FILENAME(x) x.substr(x.find_last_of('/')+1)

//! setting if statement
#ifdef NDEBUG
#define ONOFF if(false)
#else
//#define ONOFF
#define ONOFF if(false)
#endif

//! colored output for sending and receiving
#include <iostream>
#define KNX_SENDING ONOFF std::cerr << knx::COLOR_SENDING << SIMPLE_FILENAME(std::string(__FILE__)) << ":" << __LINE__ << " (" << __FUNCTION__ << ") >> " << knx::COLOR_RESET
#define KNX_RECEIVING ONOFF std::cerr << knx::COLOR_RECEIVING << SIMPLE_FILENAME(std::string(__FILE__)) << ":" << __LINE__ << " (" << __FUNCTION__ << ") >> " << knx::COLOR_RESET

//! colored output for info, warn and error messages
#define KNX_INFORMATION ONOFF std::cerr << knx::COLOR_INFO << SIMPLE_FILENAME(std::string(__FILE__)) << ":" << __LINE__ << " (" << __FUNCTION__ << ") >> " << knx::COLOR_RESET
#define KNX_WARNING ONOFF std::cerr << knx::COLOR_WARNING << SIMPLE_FILENAME(std::string(__FILE__)) << ":" << __LINE__ << " (" << __FUNCTION__ << ") WARNING >> " << knx::COLOR_RESET
#define KNX_ERROR std::cerr << knx::COLOR_ERROR << SIMPLE_FILENAME(std::string(__FILE__)) << ":" << __LINE__ << " (" << __FUNCTION__ << ") ERROR >> " << knx::COLOR_RESET
#define KNX_FINAL std::cout

// WINDOWS FOO
#ifdef __linux
#define KNX_IMPORT_EXPORT
#elif defined __APPLE__
#define KNX_IMPORT_EXPORT
#elif defined KNX_EXPORT
#define KNX_IMPORT_EXPORT __declspec(dllexport)
#else
#define KNX_IMPORT_EXPORT __declspec(dllimport)
#endif


